/* fbfont.c, derived partly from fblogo.c, wich was made by 
   Daniel Vedy <daniel@kongsberg.online.no> 1999 */

#include <magick/magick.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

int read_image(const char* path);
int write_header(const char* filename);
void process_image();

char* inputfile;
char* outputfile;
Image* image;
ImageInfo image_info;
QuantizeInfo quantize_info;


int main(int argc, char** argv)
{
    if(argc<3)
    {
        printf("Usage:\n");
        printf("%s inputfile outputfile\n", argv[0] );
        exit(EXIT_FAILURE);
    }

    inputfile=argv[1];
    outputfile=argv[2];

    if(!read_image(inputfile))
    {
        fprintf(stderr, "Unable to read image %s - %s\n", inputfile, strerror(errno));
        exit(EXIT_FAILURE);
    }
    
    process_image();
    
    if(!write_header(outputfile))
    {
        fprintf(stderr, "Unable to write header %s - %s\n", outputfile, strerror(errno));
        exit(EXIT_FAILURE);
    }
    
    printf("Generated %s successfully\n\n", outputfile);
    exit(EXIT_SUCCESS);
}

int read_image(const char* path)
{
    ExceptionInfo exception;

    GetImageInfo(&image_info);
    GetQuantizeInfo(&quantize_info);
    strcpy(image_info.filename, path);
    image = ReadImage(&image_info,&exception);
    if(image==NULL)
        return 0;
    if( image->columns != 552 || image->rows != 11 ){
        printf("Image has wrong dimensions (69 chars  8x11 pixel)\n");
	return 0;
    }
    
    return 1;
}

void process_image()
{
    quantize_info.number_colors= 2;
    QuantizeImage(&quantize_info, image);
    UncondenseImage(image);
}

int write_header(const char* path)
{
    int i,j,k;
    int fd;
    char buffer[512]={0};
    unsigned char value;
    PixelPacket *pp;
#error "This must be converted to ImageMagick5. Grrrr."
    
    fd=open(path, O_RDWR|O_CREAT|O_TRUNC, 0644);
    if(fd<0)
        return 0;
    
    sprintf(buffer, "unsigned char font[] = {\0");
    write(fd, buffer, strlen(buffer));

    for(k= 0; k<69; k++){
	write(fd, "\n       ", 8 );
        for(i=0; i < 11; i++){
            value= 0;
            for(j=0; j<8; j++){
                value+= (image->pixels[i*552+j+k*8].index==0)?(1<<j):0;
	    }

	    if( i==10 ){
		if( k==68 ) sprintf(buffer, "%d ", value);
		else sprintf(buffer, "%d ,", value);
            }
	    else sprintf(buffer, "%d ,", value);
	    write(fd, buffer, strlen(buffer));
        }
    }
    
    write(fd,"\n};\n\n", 5);

    close(fd);
    
    return 1;
}
